@@ -8,22 +8,18 @@ from typing import Callable
 
 import numpy as np
 
-from .base import SolverBase
-from ..pdes.base import PDEBase 
 from ..fields.base import FieldBase
+from ..pdes.base import PDEBase
 from ..tools.numba import jit
-
+from .base import SolverBase
 
 
 class ExplicitSolver(SolverBase):
     """ class for solving partial differential equations explicitly """
 
-    name = 'explicit'
+    name = "explicit"
 
-
-    def __init__(self, pde: PDEBase,
-                 scheme: str = 'euler',
-                 backend: str = 'auto'):
+    def __init__(self, pde: PDEBase, scheme: str = "euler", backend: str = "auto"):
         """ initialize the explicit solver
         
         Args:
@@ -36,12 +32,11 @@ class ExplicitSolver(SolverBase):
                 Determines how the function is created. Accepted  values are
                 'numpy` and 'numba'. Alternatively, 'auto' lets the code decide
                 for the most optimal backend.
-        """        
+        """
         super().__init__(pde)
         self.scheme = scheme
         self.backend = backend
-    
-    
+
     def _make_euler_stepper(self, state: FieldBase, dt: float) -> Callable:
         """ make a simple Euler stepper
         
@@ -58,17 +53,15 @@ class ExplicitSolver(SolverBase):
             `t_start` to time `t_end`. The function call signature is
             `(state: numpy.ndarray, t_start: float, t_end: float)`            
         """
-        rhs = self._make_pde_rhs(state, backend=self.backend,
-                                 allow_stochastic=True)
-        
+        rhs = self._make_pde_rhs(state, backend=self.backend, allow_stochastic=True)
+
         # obtain post-step action function
         modify_after_step = jit(self.pde.make_modify_after_step())
-        
+
         if self.pde.is_sde:
             # handle stochastic version of the pde
-            
-            def stepper(state_data: np.ndarray, t_start: float, steps: int) \
-                    -> float:
+
+            def stepper(state_data: np.ndarray, t_start: float, steps: int) -> float:
                 """ compiled inner loop for speed """
                 for i in range(steps):
                     # calculate the right hand side
@@ -78,17 +71,17 @@ class ExplicitSolver(SolverBase):
                     if noise_realization is not None:
                         state_data += np.sqrt(dt) * noise_realization
                     modify_after_step(state_data)
-                    
-                return t + dt        
 
-            self.info['stochastic'] = True
-            self._logger.info(f'Initialized explicit Euler-Maruyama stepper '
-                              'with dt=%g', dt)
-            
+                return t + dt
+
+            self.info["stochastic"] = True
+            self._logger.info(
+                f"Initialized explicit Euler-Maruyama stepper with dt=%g", dt
+            )
+
         else:
             # handle deterministic  version of the pde
-            def stepper(state_data: np.ndarray, t_start: float, steps: int) \
-                    -> float:
+            def stepper(state_data: np.ndarray, t_start: float, steps: int) -> float:
                 """ compiled inner loop for speed """
                 for i in range(steps):
                     # calculate the right hand side
@@ -97,14 +90,12 @@ class ExplicitSolver(SolverBase):
                     modify_after_step(state_data)
 
                 return t + dt
-        
-            self.info['stochastic'] = False
-            self._logger.info(f'Initialized explicit Euler stepper with dt=%g',
-                              dt)
-            
-        return stepper 
-    
-    
+
+            self.info["stochastic"] = False
+            self._logger.info(f"Initialized explicit Euler stepper with dt=%g", dt)
+
+        return stepper
+
     def _make_rk45_stepper(self, state: FieldBase, dt: float) -> Callable:
         """ make a simple Euler stepper
         
@@ -121,35 +112,31 @@ class ExplicitSolver(SolverBase):
             `t_start` to time `t_end`. The function call signature is
             `(state: numpy.ndarray, t_start: float, t_end: float)`            
         """
-        rhs = self._make_pde_rhs(state, backend=self.backend,
-                                 allow_stochastic=False)
-        self.info['stochastic'] = False
-        
+        rhs = self._make_pde_rhs(state, backend=self.backend, allow_stochastic=False)
+        self.info["stochastic"] = False
+
         # obtain post-step action function
         modify_after_step = jit(self.pde.make_modify_after_step())
-        
-        def stepper(state_data: np.ndarray, t_start: float, steps: int) \
-                -> float:
+
+        def stepper(state_data: np.ndarray, t_start: float, steps: int) -> float:
             """ compiled inner loop for speed """
             for i in range(steps):
                 # calculate the right hand side
                 t = t_start + i * dt
-                
+
                 # calculate the intermediate values in Runge-Kutta
                 k1 = dt * rhs(state_data, t)
                 k2 = dt * rhs(state_data + 0.5 * k1, t + 0.5 * dt)
                 k3 = dt * rhs(state_data + 0.5 * k2, t + 0.5 * dt)
-                k4 = dt * rhs(state_data + k3, t + dt)  
-                
+                k4 = dt * rhs(state_data + k3, t + dt)
+
                 state_data += (k1 + 2 * k2 + 2 * k3 + k4) / 6
                 modify_after_step(state_data)
 
-            return t + dt        
-        
-        self._logger.info(f'Initialized explicit Runge-Kutta-45 stepper with '
-                          'dt=%g', dt)
-        return stepper 
+            return t + dt
 
+        self._logger.info(f"Initialized explicit Runge-Kutta-45 stepper with dt=%g", dt)
+        return stepper
 
     def make_stepper(self, state: FieldBase, dt=None) -> Callable:
         """ return a stepper function using an explicit scheme
@@ -171,32 +158,28 @@ class ExplicitSolver(SolverBase):
         # the solver should use a default time step
         if dt is None:
             dt = 1e-3
-        
-        self.info['dt'] = dt
-        self.info['steps'] = 0
-        self.info['scheme'] = self.scheme
-        
-        if self.scheme == 'euler':
+
+        self.info["dt"] = dt
+        self.info["steps"] = 0
+        self.info["scheme"] = self.scheme
+
+        if self.scheme == "euler":
             inner_stepper = self._make_euler_stepper(state, dt)
-        elif self.scheme in {'runge-kutta', 'rk', 'rk45'}:
+        elif self.scheme in {"runge-kutta", "rk", "rk45"}:
             inner_stepper = self._make_rk45_stepper(state, dt)
         else:
             raise ValueError(f"Explicit scheme {self.scheme} is not supported")
 
-        if self.info['backend'] == 'numba':
+        if self.info["backend"] == "numba":
             # compile inner step
             inner_stepper = jit(inner_stepper)
-        
-        def stepper(state: FieldBase, t_start: float, t_end: float) \
-                -> float:
-            """ use Euler stepping to advance `state` from `t_start` to
-            `t_end` """
+
+        def stepper(state: FieldBase, t_start: float, t_end: float) -> float:
+            """ use Euler stepping to advance `state` from `t_start` to `t_end` """
             # calculate number of steps (which is at least 1)
             steps = max(1, int(np.ceil((t_end - t_start) / dt)))
             t_last = inner_stepper(state.data, t_start, steps)
-            self.info['steps'] += steps
-            return t_last  # type: ignore        
-            
-        return stepper
+            self.info["steps"] += steps
+            return t_last  # type: ignore
 
-        
+        return stepper
