@@ -4,26 +4,27 @@ Defines a class storing data in memory.
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de> 
 """
 
-from typing import Optional, Sequence, List
 from contextlib import contextmanager
+from typing import List, Optional, Sequence
 
 import numpy as np
 
-from .base import StorageBase, InfoDict
 from ..fields import FieldCollection
-from ..fields.base import FieldBase 
+from ..fields.base import FieldBase
+from .base import InfoDict, StorageBase
+
 
-    
-    
 class MemoryStorage(StorageBase):
     """ store discretized fields in memory """
-    
-    
-    def __init__(self, times: Optional[Sequence[float]] = None,
-                 data: Optional[List[np.ndarray]] = None,
-                 field_obj: FieldBase = None,
-                 info: InfoDict = None,
-                 write_mode: str = 'truncate_once'):
+
+    def __init__(
+        self,
+        times: Optional[Sequence[float]] = None,
+        data: Optional[List[np.ndarray]] = None,
+        field_obj: FieldBase = None,
+        info: InfoDict = None,
+        write_mode: str = "truncate_once",
+    ):
         """
         Args:
             times (:class:`numpy.ndarray`):
@@ -51,19 +52,22 @@ class MemoryStorage(StorageBase):
         self.data: List[np.ndarray] = [] if data is None else data
         if len(self.data) > 0:
             self._data_shape = self.data[0].shape
-            
+
         # check consistency
         if len(self.times) != len(self.data):
-            raise ValueError('Length of the supplied `times` and `fields` are '
-                             f'inconsistent ({len(self.times)} != '
-                             f'{len(self.data)}).')
-            
-            
+            raise ValueError(
+                "Length of the supplied `times` and `fields` are inconsistent "
+                f"({len(self.times)} != {len(self.data)})"
+            )
+
     @classmethod
-    def from_fields(cls, times: Optional[Sequence[float]] = None,
-                    fields: Optional[Sequence[FieldBase]] = None,
-                    info: InfoDict = None,
-                    write_mode: str = 'truncate_once') -> "MemoryStorage":
+    def from_fields(
+        cls,
+        times: Optional[Sequence[float]] = None,
+        fields: Optional[Sequence[FieldBase]] = None,
+        info: InfoDict = None,
+        write_mode: str = "truncate_once",
+    ) -> "MemoryStorage":
         """ create MemoryStorage from a list of fields
         
         Args:
@@ -89,16 +93,17 @@ class MemoryStorage(StorageBase):
             data = [fields[0].data]
             for field in fields[1:]:
                 if field_obj.grid != field.grid:
-                    raise ValueError('Grids of the fields are incompatible')
+                    raise ValueError("Grids of the fields are incompatible")
                 data.append(field.data)
-            
-        return cls(times, data=data, field_obj=field_obj, info=info,
-                   write_mode=write_mode)
-    
 
-    @classmethod            
-    def from_collection(cls, storages: Sequence["StorageBase"],
-                        label: str = None) -> "MemoryStorage":
+        return cls(
+            times, data=data, field_obj=field_obj, info=info, write_mode=write_mode
+        )
+
+    @classmethod
+    def from_collection(
+        cls, storages: Sequence["StorageBase"], label: str = None
+    ) -> "MemoryStorage":
         """ combine multiple memory storages into one
         
         This method can be used to combine multiple time series of different
@@ -121,25 +126,26 @@ class MemoryStorage(StorageBase):
         """
         if len(storages) == 0:
             return cls()
-        
+
         # initialize the combined data
         times = storages[0].times
         data = [[field] for field in storages[0]]
-        
+
         # append data from further storages
         for storage in storages[1:]:
             if not np.allclose(times, storage.times):
-                raise ValueError('Storages have incompatible times')
+                raise ValueError("Storages have incompatible times")
             for i, field in enumerate(storage):
                 data[i].append(field)
-                
+
         # convert data format to FieldCollections
-        fields = [FieldCollection(d, label=label)  # type: ignore
-                  for d in data]
-        
+        fields = [
+            FieldCollection(d, label=label)  # type: ignore
+            for d in data
+        ]
+
         return cls.from_fields(times, fields=fields)
-            
-        
+
     def clear(self, clear_data_shape: bool = False) -> None:
         """ truncate the storage by removing all stored data.
         
@@ -151,7 +157,6 @@ class MemoryStorage(StorageBase):
         self.data = []
         super().clear(clear_data_shape=clear_data_shape)
 
-
     def start_writing(self, field: FieldBase, info: InfoDict = None) -> None:
         """ initialize the storage for writing data
         
@@ -163,29 +168,29 @@ class MemoryStorage(StorageBase):
                 Supplies extra information that is stored in the storage
         """
         super().start_writing(field, info=info)
-        
+
         # update info after opening file because otherwise information can be
         # overwritten by data that is already present in the file
         if info is not None:
             self.info.update(info)
-            
+
         # handle the different write modes
-        if self.write_mode == 'truncate_once':
+        if self.write_mode == "truncate_once":
             self.clear()
-            self.write_mode = 'append'  # do not truncate in subsequent calls
-            
-        elif self.write_mode == 'truncate':
+            self.write_mode = "append"  # do not truncate in subsequent calls
+
+        elif self.write_mode == "truncate":
             self.clear()
 
-        elif self.write_mode == 'readonly':
-            raise RuntimeError('Cannot write in read-only mode')
-        
-        elif self.write_mode != 'append':        
-            raise ValueError(f'Unknown write mode `{self.write_mode}`. '
-                             'Possible values are `truncate_once`, '
-                             '`truncate`, and `append`')
-                    
-            
+        elif self.write_mode == "readonly":
+            raise RuntimeError("Cannot write in read-only mode")
+
+        elif self.write_mode != "append":
+            raise ValueError(
+                f"Unknown write mode `{self.write_mode}`. Possible values are "
+                "`truncate_once`, `truncate`, and `append`"
+            )
+
     def append(self, data: np.ndarray, time: Optional[float] = None) -> None:
         """ append a new data set
         
@@ -200,7 +205,6 @@ class MemoryStorage(StorageBase):
         self.times.append(time)
 
 
-
 @contextmanager
 def get_memory_storage(field: FieldBase, info: InfoDict = None):
     """ a context manager that can be used to create a MemoryStorage
