@@ -1,4 +1,4 @@
-'''
+"""
 Module containing functions for managing cache structures
 
 .. autosummary::
@@ -15,21 +15,20 @@ Module containing functions for managing cache structures
 
 
 .. codeauthor:: David Zwicker <dzwicker@seas.harvard.edu>
-'''
+"""
 
 from __future__ import division
 
 import collections
-from hashlib import sha1
 import functools
 import logging
 import numbers
-from typing import Iterable, Callable, Optional, Dict
+from hashlib import sha1
+from typing import Callable, Dict, Iterable, Optional
 
 import numpy as np
 
 
-
 def objects_equal(a, b) -> bool:
     """ compares two objects to see whether they are equal
     
@@ -48,29 +47,26 @@ def objects_equal(a, b) -> bool:
     if isinstance(b, np.ndarray):
         return np.array_equal(b, a)  # type: ignore
 
-    # compare dictionaries     
+    # compare dictionaries
     if isinstance(a, dict):
         if not isinstance(b, dict) or len(a) != len(b):
             return False
-        return all(objects_equal(v, b[k])
-                   for k, v in a.items())
-    
+        return all(objects_equal(v, b[k]) for k, v in a.items())
+
     if isinstance(a, (tuple, list)):
         if a.__class__ != b.__class__ or len(a) != len(b):
             return False
         return all(objects_equal(x, y) for x, y in zip(a, b))
-    
+
     # use direct comparison
     return a == b  # type: ignore
 
 
-
 def _hash_iter(it: Iterable) -> int:
     """ get hash of an iterable but turning it into a tuple first """
     return hash(tuple(it))
 
 
-
 def hash_mutable(obj) -> int:
     """ return hash also for (nested) mutable objects. This function might be a
     bit slow, since it iterates over all containers and hashes objects
@@ -84,31 +80,40 @@ def hash_mutable(obj) -> int:
     """
     if hasattr(obj, "_cache_hash"):
         return int(obj._cache_hash())
-    
+
     # deal with some special classes
     if isinstance(obj, (list, tuple)):
         return _hash_iter(hash_mutable(v) for v in obj)
-    
+
     if isinstance(obj, (set, frozenset)):
         return hash(frozenset(hash_mutable(v) for v in obj))
-    
+
     if isinstance(obj, collections.OrderedDict):
-        return _hash_iter((k, hash_mutable(v))
-                          for k, v in obj.items()
-                          if not (isinstance(k, str) and
-                                  k.startswith('_cache')))
-    
-    if isinstance(obj, (dict, collections.abc.MutableMapping,
-                        collections.defaultdict, collections.Counter)):
-        return hash(frozenset((k, hash_mutable(v))
-                              for k, v in sorted(obj.items())
-                              if not (isinstance(k, str) and
-                                      k.startswith('_cache'))))
-    
+        return _hash_iter(
+            (k, hash_mutable(v))
+            for k, v in obj.items()
+            if not (isinstance(k, str) and k.startswith("_cache"))
+        )
+
+    unordered_mappings = (
+        dict,
+        collections.abc.MutableMapping,
+        collections.defaultdict,
+        collections.Counter,
+    )
+    if isinstance(obj, unordered_mappings):
+        return hash(
+            frozenset(
+                (k, hash_mutable(v))
+                for k, v in sorted(obj.items())
+                if not (isinstance(k, str) and k.startswith("_cache"))
+            )
+        )
+
     if isinstance(obj, np.ndarray):
         return hash(obj.tobytes())
-    
-    try:    
+
+    try:
         # try using the internal hash function
         return hash(obj)
     except TypeError:
@@ -118,9 +123,8 @@ def hash_mutable(obj) -> int:
         except (ValueError, TypeError):
             # otherwise, hash the internal dict
             return hash_mutable(obj.__dict__)
-    
-    
-    
+
+
 def hash_readable(obj) -> str:
     """ return human readable hash also for (nested) mutable objects. This
     function returns a json-like representation of the object. The function
@@ -137,45 +141,53 @@ def hash_readable(obj) -> str:
     """
     if isinstance(obj, numbers.Number):
         return str(obj)
-    
+
     if isinstance(obj, (str, bytes)):
-        return '"' + str(obj).replace('\\', '\\\\').replace('"', '\"') + '"'
-    
+        return '"' + str(obj).replace("\\", "\\\\").replace('"', '"') + '"'
+
     if isinstance(obj, (list, tuple)):
-        return '[' + ', '.join(hash_readable(v) for v in obj) + ']'
-    
+        return "[" + ", ".join(hash_readable(v) for v in obj) + "]"
+
     if isinstance(obj, (set, frozenset)):
-        return '{' + ', '.join(hash_readable(v) for v in sorted(obj)) + '}'
-    
-    if isinstance(obj, (dict, collections.abc.MutableMapping,
-                        collections.OrderedDict, collections.defaultdict,
-                        collections.Counter)):
-        return '{' + ', '.join(hash_readable(k) + ': ' + hash_readable(v)
-                               for k, v in sorted(obj.items())) + '}'
-                               
+        return "{" + ", ".join(hash_readable(v) for v in sorted(obj)) + "}"
+
+    mappings = (
+        dict,
+        collections.abc.MutableMapping,
+        collections.OrderedDict,
+        collections.defaultdict,
+        collections.Counter,
+    )
+    if isinstance(obj, mappings):
+        hash_str = ", ".join(
+            hash_readable(k) + ": " + hash_readable(v) for k, v in sorted(obj.items())
+        )
+        return "{" + hash_str + "}"
+
     if isinstance(obj, np.ndarray):
         return repr(obj)
-    
+
     # otherwise, assume it's a generic object
     try:
-        if hasattr(obj, '__getstate__'):
+        if hasattr(obj, "__getstate__"):
             data = obj.__getstate__()
         else:
             data = obj.__dict__
-            
+
     except AttributeError:
         # strange object without a dictionary attached to it
         return repr(obj)
-    
+
     else:
         # turn arguments into something readable
-        args = ', '.join(str(k) + '=' + hash_readable(v)
-                         for k, v in sorted(data.items())
-                         if not k.startswith('_'))
-    
-        return '{name}({args})'.format(name=obj.__class__.__name__, args=args)
-    
-    
+        args = ", ".join(
+            str(k) + "=" + hash_readable(v)
+            for k, v in sorted(data.items())
+            if not k.startswith("_")
+        )
+
+        return "{name}({args})".format(name=obj.__class__.__name__, args=args)
+
 
 def make_serializer(method: str) -> Callable:
     """ returns a function that serialize data with the given method. Note that
@@ -190,33 +202,35 @@ def make_serializer(method: str) -> Callable:
     """
     if callable(method):
         return method
-    
+
     if method is None:
         return lambda s: s
-    
-    if method == 'hash':
-        return hash
-    
-    if method == 'hash_mutable':
-        return hash_mutable
-    
-    if method == 'hash_readable':
-        return hash_readable
-    
-    if method == 'json':
-        import json
-        return lambda s: json.dumps(s, sort_keys=True).encode('utf-8')
 
-    if method == 'pickle':
+    if method == "hash":
+        return hash
+
+    if method == "hash_mutable":
+        return hash_mutable
+
+    if method == "hash_readable":
+        return hash_readable
+
+    if method == "json":
+        import json
+
+        return lambda s: json.dumps(s, sort_keys=True).encode("utf-8")
+
+    if method == "pickle":
         import pickle
+
         return lambda s: pickle.dumps(s, protocol=pickle.HIGHEST_PROTOCOL)
 
-    if method == 'yaml':
+    if method == "yaml":
         import yaml
-        return lambda s: yaml.dump(s).encode('utf-8')
-    
-    raise ValueError('Unknown serialization method `%s`' % method)
 
+        return lambda s: yaml.dump(s).encode("utf-8")
+
+    raise ValueError("Unknown serialization method `%s`" % method)
 
 
 def make_unserializer(method: str) -> Callable:
@@ -238,59 +252,56 @@ def make_unserializer(method: str) -> Callable:
     if method is None:
         return lambda s: s
 
-    if method == 'json':
+    if method == "json":
         import json
-        return lambda s: json.loads(s.decode('utf-8'))
 
-    if method == 'pickle':
+        return lambda s: json.loads(s.decode("utf-8"))
+
+    if method == "pickle":
         import pickle
+
         return lambda s: pickle.loads(s)
 
-    if method == 'yaml':
+    if method == "yaml":
         import yaml
+
         return yaml.full_load
-    
-    if method == 'yaml_unsafe':
+
+    if method == "yaml_unsafe":
         import yaml  # @Reimport
+
         return yaml.unsafe_load  # type: ignore
-    
-    raise ValueError('Unknown serialization method `%s`' % method)
-            
-            
+
+    raise ValueError("Unknown serialization method `%s`" % method)
+
 
 class DictFiniteCapacity(collections.OrderedDict):
     """ cache with a limited number of items """
-    
-    default_capacity: int = 100
-    
-    def __init__(self, *args, **kwargs):
-        self.capacity = kwargs.pop('capacity', self.default_capacity)
-        super(DictFiniteCapacity, self).__init__(*args, **kwargs)
 
+    default_capacity: int = 100
+
+    def __init__(self, *args, **kwargs):
+        self.capacity = kwargs.pop("capacity", self.default_capacity)
+        super(DictFiniteCapacity, self).__init__(*args, **kwargs)
 
     def check_length(self):
         """ ensures that the dictionary does not grow beyond its capacity """
         while len(self) > self.capacity:
             self.popitem(last=False)
-            
-            
+
     def __eq__(self, other):
         return super().__eq__(other) and self.capacity == other.capacity
-           
-            
+
     def __ne__(self, other):
         return super().__ne__(other) or self.capacity != other.capacity
-            
 
     def __setitem__(self, key, value):
         super(DictFiniteCapacity, self).__setitem__(key, value)
         self.check_length()
-        
-        
+
     def update(self, values):
         super(DictFiniteCapacity, self).update(values)
         self.check_length()
-        
 
 
 class SerializedDict(collections.abc.MutableMapping):
@@ -298,10 +309,13 @@ class SerializedDict(collections.abc.MutableMapping):
     This class provides hooks for converting arbitrary keys and values to
     strings, which are then stored in the database.
     """
-    
-    def __init__(self, key_serialization: str = 'pickle',
-                 value_serialization: str = 'pickle',
-                 storage_dict: Optional[Dict] = None):
+
+    def __init__(
+        self,
+        key_serialization: str = "pickle",
+        value_serialization: str = "pickle",
+        storage_dict: Optional[Dict] = None,
+    ):
         """ provides a dictionary whose keys and values are serialized
         
         Args:
@@ -318,18 +332,16 @@ class SerializedDict(collections.abc.MutableMapping):
             self._data = {}
         else:
             self._data = storage_dict
-        
+
         # define the methods that serialize and unserialize the data
         self.serialize_key = make_serializer(key_serialization)
         self.unserialize_key = make_unserializer(key_serialization)
         self.serialize_value = make_serializer(value_serialization)
         self.unserialize_value = make_unserializer(value_serialization)
-    
-    
+
     def __len__(self):
         return len(self._data)
-    
-    
+
     def __getitem__(self, key):
         # convert key to its string representation
         key_s = self.serialize_key(key)
@@ -337,8 +349,7 @@ class SerializedDict(collections.abc.MutableMapping):
         value = self._data[key_s]
         # convert the value to its object representation
         return self.unserialize_value(value)
-        
-        
+
     def __setitem__(self, key, value):
         # convert key and value to their string representations
         key_s = self.serialize_key(key)
@@ -346,21 +357,18 @@ class SerializedDict(collections.abc.MutableMapping):
         # add the item to the dictionary
         self._data[key_s] = value_s
 
-
     def __delitem__(self, key):
         # convert key to its string representation
         key_s = self.serialize_key(key)
         # delete the item from the dictionary
         del self._data[key_s]
-    
-    
+
     def __contains__(self, key):
         # convert key to its string representation
         key_s = self.serialize_key(key)
         # check whether this items exists in the dictionary
         return key_s in self._data
-    
-    
+
     def __iter__(self):
         # iterate  dictionary
         for key_s in self._data.__iter__():
@@ -368,12 +376,18 @@ class SerializedDict(collections.abc.MutableMapping):
             yield self.unserialize_key(key_s)
 
 
-
 class _class_cache:
     """ class handling the caching of results of methods and properties """
-    
-    def __init__(self, factory=None, extra_args=None, ignore_args=None,
-                 hash_function='hash_mutable', doc=None, name=None):
+
+    def __init__(
+        self,
+        factory=None,
+        extra_args=None,
+        ignore_args=None,
+        hash_function="hash_mutable",
+        doc=None,
+        name=None,
+    ):
         r""" decorator that caches calls in a dictionary attached to the
         instances. This can be used with most classes
     
@@ -461,23 +475,24 @@ class _class_cache:
         # check whether the decorator has been applied correctly
         if callable(factory):
             class_name = self.__class__.__name__
-            raise ValueError('Missing function call. Call this decorator as '
-                             '@{0}() instead of @{0}'.format(class_name))
-            
+            raise ValueError(
+                f"Missing function call. Call this decorator as {class_name}() instead "
+                f"of {class_name}"
+            )
+
         else:
             self.factory = factory
-        
-    
+
     def _get_clear_cache_method(self) -> Callable:
         """ return a method that can be attached to classes to clear the cache
         of the wrapped method """
-        
+
         def clear_cache(obj):
             """ clears the cache associated with this method """
             try:
                 # try getting an initialized cache
                 cache = obj._cache_methods[self.name]
-                
+
             except (AttributeError, KeyError):
                 # the cache was not initialized
                 if self.factory is None:
@@ -487,22 +502,21 @@ class _class_cache:
                 # initialize the cache, since it might open a persistent
                 # database, which needs to be cleared
                 cache = getattr(obj, self.factory)(self.name)
-                
+
             # clear the cache
-            cache.clear()   
-            
+            cache.clear()
+
         return clear_cache
-    
-        
+
     def _get_wrapped_function(self, func: Callable) -> Callable:
         """ return the wrapped method, which implements the cache """
-        
+
         if self.name is None:
             self.name = func.__name__
-    
+
         # create the function to serialize the keys
         hash_key = make_serializer(self.hash_function)
-    
+
         @functools.wraps(func)
         def wrapper(obj, *args, **kwargs):
             # try accessing the cache
@@ -510,7 +524,7 @@ class _class_cache:
                 cache = obj._cache_methods[self.name]
             except (AttributeError, KeyError) as err:
                 # the cache was not initialized
-                wrapper._logger.debug('Initialize the cache `%s`', self.name)
+                wrapper._logger.debug("Initialize the cache `%s`", self.name)
                 if isinstance(err, AttributeError):
                     # the cache dictionary is not even present
                     obj._cache_methods = {}
@@ -521,37 +535,38 @@ class _class_cache:
                     cache = getattr(obj, self.factory)(self.name)
                 # store the cache in the dictionary
                 obj._cache_methods[self.name] = cache
-    
+
             # determine the key that encodes the current arguments
             if self.ignore_args:
-                kwargs_key = {k: v for k, v in kwargs.items()
-                              if k not in self.ignore_args}
+                kwargs_key = {
+                    k: v for k, v in kwargs.items() if k not in self.ignore_args
+                }
                 func_args = [args, kwargs_key]
             else:
                 func_args = [args, kwargs]
-                
+
             if self.extra_args:
                 for extra_arg in self.extra_args:
                     func_args.append(getattr(obj, extra_arg))
-                    
+
             cache_key = hash_key(tuple(func_args))
-    
+
             try:
                 # try loading the results from the cache
                 result = cache[cache_key]
             except KeyError:
                 # if this failed, compute and store the results
-                wrapper._logger.debug('Cache missed. Compute result for method '
-                                      '`%s`', self.name)
+                wrapper._logger.debug(
+                    "Cache missed. Compute result for method `%s`", self.name
+                )
                 result = func(obj, *args, **kwargs)
                 cache[cache_key] = result
             return result
-        
+
         # initialize the logger
         wrapper._logger = logging.getLogger(__name__)  # type: ignore
-        
-        return wrapper
 
+        return wrapper
 
 
 class cached_property(_class_cache):
@@ -588,20 +603,15 @@ class cached_property(_class_cache):
         self._cache_name = self.name
         self.clear_cache_of_obj = self._get_clear_cache_method()
         self.func = self._get_wrapped_function(method)
-    
+
         self.__doc__ = self.func.__doc__
         self.__name__ = self.func.__name__
         self.__module__ = self.func.__module__
         return self
-    
-    
+
     def __get__(self, obj, owner):
         """ call the method to obtain the result for this property """
-#         if obj is None:
-#             return self
-
         return self.func(obj)
-    
 
 
 class cached_method(_class_cache):
@@ -629,14 +639,14 @@ class cached_method(_class_cache):
     cleared using `self.\_cache\_methods[property\_name] = {}`, where 
     `property\_name` is the name of the property
     """
-    
+
     def __call__(self, method):
         """ apply the cache decorator to the method """
-        
+
         wrapper = self._get_wrapped_function(method)
-    
+
         # save name, e.g., to be able to delete cache later
         wrapper._cache_name = self.name
         wrapper.clear_cache_of_obj = self._get_clear_cache_method()
-    
+
         return wrapper
