@@ -72,20 +72,18 @@ harmonics, where the order is always zero and the degree :math:`l` and the mode
 """
 
 import itertools
-from typing import Tuple, Callable, Optional
+from typing import Callable, Optional, Tuple
 
 import numpy as np
 from scipy import spatial
 from scipy.special import sph_harm
 
-from ..tools.numba import jit
 from ..tools.cache import cached_method
-
+from ..tools.numba import jit
 
 π = np.pi
 
 
-
 def radius_from_volume(volume: float, dim: int) -> float:
     """ Return the radius of a sphere with a given volume
     
@@ -101,11 +99,9 @@ def radius_from_volume(volume: float, dim: int) -> float:
     elif dim == 2:
         return np.sqrt(volume / π)  # type: ignore
     elif dim == 3:
-        return (3 * volume / (4 * π))**(1/3)  # type: ignore
+        return (3 * volume / (4 * π)) ** (1 / 3)  # type: ignore
     else:
-        raise NotImplementedError(f'Cannot calculate the radius in {dim} '
-                                  'dimensions')
-
+        raise NotImplementedError(f"Cannot calculate the radius in {dim} dimensions")
 
 
 def make_radius_from_volume_compiled(dim: int) -> Callable:
@@ -118,19 +114,23 @@ def make_radius_from_volume_compiled(dim: int) -> Callable:
         function: A function that takes a volume and returns the radius
     """
     if dim == 1:
+
         def radius_from_volume(volume):
             return volume / 2
+
     elif dim == 2:
+
         def radius_from_volume(volume):
             return np.sqrt(volume / π)
-    elif dim == 3:
-        def radius_from_volume(volume):
-            return (3 * volume / (4 * π))**(1/3)
-    else:
-        raise NotImplementedError(f'Cannot calculate the radius in {dim} '
-                                  'dimensions')
-    return jit(radius_from_volume)  # type: ignore
 
+    elif dim == 3:
+
+        def radius_from_volume(volume):
+            return (3 * volume / (4 * π)) ** (1 / 3)
+
+    else:
+        raise NotImplementedError(f"Cannot calculate the radius in {dim} dimensions")
+    return jit(radius_from_volume)  # type: ignore
 
 
 def volume_from_radius(radius: float, dim: int) -> float:
@@ -146,13 +146,11 @@ def volume_from_radius(radius: float, dim: int) -> float:
     if dim == 1:
         return 2 * radius
     elif dim == 2:
-        return π * radius**2  # type: ignore
+        return π * radius ** 2  # type: ignore
     elif dim == 3:
-        return 4 * π / 3 * radius**3  # type: ignore
+        return 4 * π / 3 * radius ** 3  # type: ignore
     else:
-        raise NotImplementedError(f'Cannot calculate the volume in {dim} '
-                                  'dimensions')
-
+        raise NotImplementedError(f"Cannot calculate the volume in {dim} dimensions")
 
 
 def make_volume_from_radius_compiled(dim: int) -> Callable:
@@ -165,19 +163,23 @@ def make_volume_from_radius_compiled(dim: int) -> Callable:
         function: A function that takes a radius and returns the volume
     """
     if dim == 1:
+
         def volume_from_radius(radius):
             return 2 * radius
-    elif dim == 2:
-        def volume_from_radius(radius):
-            return π * radius**2
-    elif dim == 3:
-        def volume_from_radius(radius):
-            return 4 * π / 3 * radius**3
-    else:
-        raise NotImplementedError(f'Cannot calculate the volume in {dim} '
-                                  'dimensions')
-    return jit(volume_from_radius)  # type: ignore
 
+    elif dim == 2:
+
+        def volume_from_radius(radius):
+            return π * radius ** 2
+
+    elif dim == 3:
+
+        def volume_from_radius(radius):
+            return 4 * π / 3 * radius ** 3
+
+    else:
+        raise NotImplementedError(f"Cannot calculate the volume in {dim} dimensions")
+    return jit(volume_from_radius)  # type: ignore
 
 
 def surface_from_radius(radius: float, dim: int) -> float:
@@ -195,11 +197,11 @@ def surface_from_radius(radius: float, dim: int) -> float:
     elif dim == 2:
         return 2 * π * radius  # type: ignore
     elif dim == 3:
-        return 4 * π * radius**2  # type: ignore
+        return 4 * π * radius ** 2  # type: ignore
     else:
-        raise NotImplementedError(f'Cannot calculate the surface area in {dim} '
-                                  'dimensions')
-
+        raise NotImplementedError(
+            f"Cannot calculate the surface area in {dim} dimensions"
+        )
 
 
 def radius_from_surface(surface: float, dim: int) -> float:
@@ -213,15 +215,13 @@ def radius_from_surface(surface: float, dim: int) -> float:
         float: Radius of the sphere
     """
     if dim == 1:
-        raise RuntimeError('Cannot calculate radius of 1-d sphere from surface')
+        raise RuntimeError("Cannot calculate radius of 1-d sphere from surface")
     elif dim == 2:
         return surface / (2 * π)  # type: ignore
     elif dim == 3:
         return np.sqrt(surface / (4 * π))  # type: ignore
     else:
-        raise NotImplementedError(f'Cannot calculate the radius in {dim} '
-                                  'dimensions')
-
+        raise NotImplementedError(f"Cannot calculate the radius in {dim} dimensions")
 
 
 def make_surface_from_radius_compiled(dim: int) -> Callable:
@@ -234,19 +234,25 @@ def make_surface_from_radius_compiled(dim: int) -> Callable:
         function: A function that takes a radius and returns the surface area
     """
     if dim == 1:
+
         def surface_from_radius(radius):
             return 2
+
     elif dim == 2:
+
         def surface_from_radius(radius):
             return 2 * π * radius
-    elif dim == 3:
-        def surface_from_radius(radius):
-            return 4 * π * radius**2
-    else:
-        raise NotImplementedError(f'Cannot calculate the surface area in {dim} '
-                                  'dimensions')
-    return jit(surface_from_radius)  # type: ignore
 
+    elif dim == 3:
+
+        def surface_from_radius(radius):
+            return 4 * π * radius ** 2
+
+    else:
+        raise NotImplementedError(
+            f"Cannot calculate the surface area in {dim} dimensions"
+        )
+    return jit(surface_from_radius)  # type: ignore
 
 
 def points_cartesian_to_spherical(points):
@@ -260,7 +266,7 @@ def points_cartesian_to_spherical(points):
     """
     points = np.atleast_1d(points)
     assert points.shape[-1] == 3
-    
+
     ps_spherical = np.empty(points.shape)
     # calculate radius in [0, infinty]
     ps_spherical[..., 0] = np.linalg.norm(points, axis=-1)
@@ -271,7 +277,6 @@ def points_cartesian_to_spherical(points):
     return ps_spherical
 
 
-
 def points_spherical_to_cartesian(points):
     """ Convert points from spherical to Cartesian coordinates 
     
@@ -284,7 +289,7 @@ def points_spherical_to_cartesian(points):
     """
     points = np.atleast_1d(points)
     assert points.shape[-1] == 3
-    
+
     sin_θ = np.sin(points[..., 1])
     ps_cartesian = np.empty(points.shape)
     ps_cartesian[..., 0] = points[..., 0] * np.cos(points[..., 2]) * sin_θ
@@ -293,7 +298,6 @@ def points_spherical_to_cartesian(points):
     return ps_cartesian
 
 
-
 def haversine_distance(point1, point2) -> float:
     """ Calculate the haversine-based distance between two points on the surface
     of a sphere. Should be more accurate than the arc cosine strategy.
@@ -309,12 +313,12 @@ def haversine_distance(point1, point2) -> float:
         radius (float): Radius of the sphere
     """
     # note that latitude φ is θ and longitude λ is φ in our notation
-    r1, φ1, λ1 = points_cartesian_to_spherical(point1)  
+    r1, φ1, λ1 = points_cartesian_to_spherical(point1)
     r2, φ2, λ2 = points_cartesian_to_spherical(point2)
-    
+
     # check whether both points lie on the same sphere
     assert np.isclose(r1, r2)
-    
+
     # we rewrite the standard Haversine slightly as long/lat is not the same as
     # spherical coordinates - φ differs by π/4
     factor = (1 - np.cos(λ2 - λ1)) / 2
@@ -322,7 +326,6 @@ def haversine_distance(point1, point2) -> float:
     return 2 * r1 * np.arcsin(np.sqrt(arg))  # type: ignore
 
 
-
 def get_spherical_polygon_area(vertices, radius: float = 1) -> float:
     """ Calculate the surface area of a polygon on the surface of a sphere.
     Based on equation provided here:
@@ -342,33 +345,36 @@ def get_spherical_polygon_area(vertices, radius: float = 1) -> float:
     spherical_coordinates = points_cartesian_to_spherical(vertices)
     spherical_coordinates[..., 0] = 1.0
     vertices = points_spherical_to_cartesian(spherical_coordinates)
-    
+
     n = vertices.shape[0]
     # point we start from
     root_point = vertices[0]
     totalexcess = 0
-    
+
     # loop from 1 to n-2, with point 2 to n-1 as other vertex of triangle
     # this could definitely be written more nicely
     b_point = vertices[1]
     root_b_dist = haversine_distance(root_point, b_point)
     for i in np.arange(1, n - 1):
         a_point = b_point
-        b_point = vertices[i+1]
+        b_point = vertices[i + 1]
         root_a_dist = root_b_dist
         root_b_dist = haversine_distance(root_point, b_point)
         a_b_dist = haversine_distance(a_point, b_point)
-        s = (root_a_dist + root_b_dist + a_b_dist) / 2.
-        arg = np.tan(0.5 * s) * np.tan(0.5 * (s - root_a_dist)) * \
-              np.tan(0.5 * (s - root_b_dist)) * np.tan(0.5 * (s - a_b_dist))
+        s = (root_a_dist + root_b_dist + a_b_dist) / 2.0
+        arg = (
+            np.tan(0.5 * s)
+            * np.tan(0.5 * (s - root_a_dist))
+            * np.tan(0.5 * (s - root_b_dist))
+            * np.tan(0.5 * (s - a_b_dist))
+        )
         totalexcess += 4 * np.arctan(np.sqrt(arg))
     return totalexcess * radius ** 2
 
 
-
-class PointsOnSphere():
+class PointsOnSphere:
     """ class representing points on an n-dimensional unit sphere """
-    
+
     def __init__(self, points):
         """
         Args:
@@ -379,8 +385,7 @@ class PointsOnSphere():
         # normalize vectors to force them onto the unit-sphere
         self.points /= np.linalg.norm(self.points, axis=1)[:, np.newaxis]
         self.dim = self.points.shape[-1]
-        
-        
+
     @classmethod
     def make_uniform(cls, dim: int, num_points: Optional[int] = None):
         """ create uniformly distributed points on a sphere
@@ -398,14 +403,14 @@ class PointsOnSphere():
             if num_points != 2:
                 raise ValueError("Can only place 2 points in 1d")
             points = [[-1], [1]]
-        
+
         elif dim == 2:
             if num_points is None:
                 num_points = 8
             # distribute points evenly around the circle
-            φs = np.linspace(0, 2*π, num_points, endpoint=False)
-            points = np.c_[np.cos(φs), np.sin(φs)] 
-            
+            φs = np.linspace(0, 2 * π, num_points, endpoint=False)
+            points = np.c_[np.cos(φs), np.sin(φs)]
+
         elif dim == 3:
             # Distribute points on the unit sphere using a sunflower spiral
             # (inspired by https://stackoverflow.com/a/44164075/932593)
@@ -413,26 +418,25 @@ class PointsOnSphere():
                 num_points = 18
             indices = np.arange(0, num_points) + 0.5
             φ = np.arccos(1 - 2 * indices / num_points)
-            θ = π * (1 + 5**0.5) * indices
-            
+            θ = π * (1 + 5 ** 0.5) * indices
+
             # convert to Cartesian coordinates
-            points = np.c_[np.cos(θ) * np.sin(φ),
-                           np.sin(θ) * np.sin(φ),
-                           np.cos(φ)]
-        
+            points = np.c_[np.cos(θ) * np.sin(φ), np.sin(θ) * np.sin(φ), np.cos(φ)]
+
         elif num_points is None:
             # use vertices of hypercube in n dimensions
-            points = [p  # type: ignore
-                      for p in itertools.product([-1, 0, 1], repeat=dim)
-                      if any(c != 0 for c in p)]
-            
+            points = [
+                p  # type: ignore
+                for p in itertools.product([-1, 0, 1], repeat=dim)
+                if any(c != 0 for c in p)
+            ]
+
         else:
             raise NotImplementedError()
-            
+
         # normalize vectors
         return cls(points)
-    
-    
+
     @cached_method()
     def get_area_weights(self, balance_axes: bool = True):
         """ return the weight of each point associated with the unit cell size
@@ -448,7 +452,7 @@ class PointsOnSphere():
         points_flat = self.points.reshape(-1, self.dim)
         if self.dim == 1:
             weights = np.array([0.5, 0.5])
-        
+
         elif self.dim == 2:
             # get angles
             φ = np.arctan2(points_flat[:, 1], points_flat[:, 0])
@@ -457,20 +461,25 @@ class PointsOnSphere():
             sizes = np.r_[s0, np.diff(φ[idx]), s0]
             weights = (sizes[1:] + sizes[:-1]) / 2
             weights /= 2 * π
-            
+
         elif self.dim == 3:
             # calculate weights using spherical voronoi construction
             voronoi = spatial.SphericalVoronoi(points_flat)
             voronoi.sort_vertices_of_regions()
-                  
-            weights = np.array([get_spherical_polygon_area(voronoi.vertices[ix])
-                                for ix in voronoi.regions], dtype=np.double)
+
+            weights = np.array(
+                [
+                    get_spherical_polygon_area(voronoi.vertices[ix])
+                    for ix in voronoi.regions
+                ],
+                dtype=np.double,
+            )
             weights /= surface_from_radius(1, dim=self.dim)
-            
+
         else:
             raise NotImplementedError()
 
-        if balance_axes:        
+        if balance_axes:
             weights /= weights.sum()  # normalize weights
             # adjust weights such that all distances are weighted equally, i.e.,
             # the weighted sum of all shell vectors should vanish. Additionally,
@@ -480,10 +489,9 @@ class PointsOnSphere():
             matrix = np.c_[points_flat, np.ones(len(points_flat))]
             vector = -weights @ matrix + np.r_[np.zeros(self.dim), 1]
             weights += np.linalg.lstsq(matrix.T, vector, rcond=None)[0]
-        
+
         return weights.reshape(self.points.shape[:-1])
-    
-    
+
     def get_distance_matrix(self):
         """ calculate the (spherical) distances between each point
         
@@ -491,35 +499,33 @@ class PointsOnSphere():
             :class:`numpy.ndarray`: the distance of each point to each other
         """
         if self.dim == 1:
-            raise ValueError('Distances can only be calculated for dim >= 2')
-            
+            raise ValueError("Distances can only be calculated for dim >= 2")
+
         elif self.dim == 2:
             # use arc length on unit circle to calculate distances
             def metric(a, b):
                 return np.arccos(a @ b)
-            
+
         elif self.dim == 3:
             # calculate distances on sphere using haversine definition
             metric = haversine_distance
-            
+
         else:
             raise NotImplementedError()
-        
+
         # determine the distances between all points
         dists = spatial.distance.pdist(self.points, metric)
-        return spatial.distance.squareform(dists)    
-    
-    
+        return spatial.distance.squareform(dists)
+
     def get_mean_separation(self) -> float:
         """ float: calculates the mean distance to the nearest neighbor """
         if len(self.points) < 1:
-            return float('nan')
-        
+            return float("nan")
+
         dists_sorted = np.sort(self.get_distance_matrix(), axis=1)
         return float(dists_sorted[:, 1].mean())
-        
-            
-    def write_to_xyz(self, path: str, comment: str = '', symbol: str = 'S'):
+
+    def write_to_xyz(self, path: str, comment: str = "", symbol: str = "S"):
         """ write the point coordinates to a xyz file
         
         Args:
@@ -527,14 +533,13 @@ class PointsOnSphere():
             comment (str, optional): comment that is written to the second line
             symbol (str, optional): denotes the symbol used for the atoms
         """
-        with open(path, 'w') as fp:
-            fp.write('%d\n' % len(self.points))
-            fp.write(comment + '\n')
+        with open(path, "w") as fp:
+            fp.write("%d\n" % len(self.points))
+            fp.write(comment + "\n")
             for point in self.points:
-                point_str = ' '.join(('%.12g' % v for v in point))
-                line = '%s %s\n' % (symbol, point_str)
+                point_str = " ".join(("%.12g" % v for v in point))
+                line = "%s %s\n" % (symbol, point_str)
                 fp.write(line)
-            
 
 
 def spherical_index_k(degree: int, order: int = 0) -> int:
@@ -551,9 +556,8 @@ def spherical_index_k(degree: int, order: int = 0) -> int:
         int: a combined index k
     """
     if not -degree <= order <= degree:
-        raise ValueError('order must lie between -degree and degree')
-    return degree*(degree + 1) + order
-
+        raise ValueError("order must lie between -degree and degree")
+    return degree * (degree + 1) + order
 
 
 def spherical_index_lm(k: int) -> Tuple[int, int]:
@@ -567,8 +571,7 @@ def spherical_index_lm(k: int) -> Tuple[int, int]:
         assoicated with the combined index      
     """
     degree = int(np.floor(np.sqrt(k)))
-    return degree, k - degree*(degree + 1)
-
+    return degree, k - degree * (degree + 1)
 
 
 def spherical_index_count(l: int) -> int:
@@ -585,16 +588,14 @@ def spherical_index_count(l: int) -> int:
     return 1 + 2 * l + l * l
 
 
-
 def spherical_index_count_optimal(k_count: int) -> bool:
     """ checks whether the modes captures all orders for maximal degree
     
     Args:
         k_count (int): The number of modes considered
     """
-    is_square = bool(int(np.sqrt(k_count) + 0.5)**2 == k_count)
-    return is_square 
-
+    is_square = bool(int(np.sqrt(k_count) + 0.5) ** 2 == k_count)
+    return is_square
 
 
 def spherical_harmonic_symmetric(degree: int, θ: float) -> float:
@@ -610,12 +611,10 @@ def spherical_harmonic_symmetric(degree: int, θ: float) -> float:
     """
     # note that the definition of `sph_harm` has a different convention for the
     # usage of the variables φ and θ and we thus have to swap the args
-    return np.real(sph_harm(0., degree, 0., θ))  # type: ignore
+    return np.real(sph_harm(0.0, degree, 0.0, θ))  # type: ignore
 
 
-
-def spherical_harmonic_real(degree: int, order: int,
-                            θ: float, φ: float) -> float:
+def spherical_harmonic_real(degree: int, order: int, θ: float, φ: float) -> float:
     r""" real spherical harmonics of degree l and order m
     
     Args:
@@ -633,18 +632,17 @@ def spherical_harmonic_real(degree: int, order: int,
     # usage of the variables φ and θ and we thus have to swap the args
     # Moreover, the scipy functions expect first the order and then the degree
     if order > 0:
-        return np.real((sph_harm(order, degree, φ, θ)  # type: ignore
-                        + (-1)**order * sph_harm(-order, degree, φ, θ)
-                        )/np.sqrt(2))
-        
+        term1 = sph_harm(order, degree, φ, θ)
+        term2 = (-1) ** order * sph_harm(-order, degree, φ, θ)
+        return np.real((term1 + term2) / np.sqrt(2))  # type: ignore
+
     elif order == 0:
         return np.real(sph_harm(0, degree, φ, θ))  # type: ignore
-    
-    else:  # order < 0
-        return np.real((sph_harm(-order, degree, φ, θ)  # type: ignore
-                        - (-1)**order * sph_harm(order, degree, φ, θ)
-                        )/(np.complex(0, np.sqrt(2))))
 
+    else:  # order < 0
+        term1 = sph_harm(-order, degree, φ, θ)
+        term2 = (-1) ** order * sph_harm(order, degree, φ, θ)
+        return np.real((term1 - term2) / (np.complex(0, np.sqrt(2))))  # type: ignore
 
 
 def spherical_harmonic_real_k(k: int, θ: float, φ: float) -> float:
@@ -662,4 +660,3 @@ def spherical_harmonic_real_k(k: int, θ: float, φ: float) -> float:
         float: The value of the spherical harmonics
     """
     return spherical_harmonic_real(*spherical_index_lm(k), θ=θ, φ=φ)
-
