@@ -20,38 +20,35 @@ The trackers defined in this module are:
 """
 
 import inspect
-import sys
 import os.path
+import sys
 import time
 from datetime import timedelta
 from pathlib import Path
-from typing import (Callable, Optional, Union, IO, List, Any,  # @UnusedImport
-                    Dict, TYPE_CHECKING)
+from typing import List  # @UnusedImport
+from typing import IO, TYPE_CHECKING, Any, Callable, Dict, Optional, Union
 
 import numpy as np
 
-from .base import TrackerBase, InfoDict, FinishedSimulation, Real
-from .intervals import IntervalData, RealtimeIntervals
-from ..fields.base import FieldBase
 from ..fields import FieldCollection
-from ..tools.parse_duration import parse_duration
-from ..tools.misc import get_progress_bar_class
+from ..fields.base import FieldBase
 from ..tools.docstrings import fill_in_docstring
-
-
+from ..tools.misc import get_progress_bar_class
+from ..tools.parse_duration import parse_duration
+from .base import FinishedSimulation, InfoDict, Real, TrackerBase
+from .intervals import IntervalData, RealtimeIntervals
 
 if TYPE_CHECKING:
     import pandas  # @UnusedImport
-    from ..visualization.movies import Movie  # @UnusedImport
 
+    from ..visualization.movies import Movie  # @UnusedImport
 
 
 class CallbackTracker(TrackerBase):
     """ Tracker that calls a function periodically """
-    
+
     @fill_in_docstring
-    def __init__(self, func: Callable,
-                 interval: IntervalData = 1):
+    def __init__(self, func: Callable, interval: IntervalData = 1):
         """ 
         Args:
             func: The function to call periodically. The function signature
@@ -68,10 +65,11 @@ class CallbackTracker(TrackerBase):
         self._callback = func
         self._num_args = len(inspect.signature(func).parameters)
         if not 0 < self._num_args < 3:
-            raise ValueError('`func` must be a function accepting one or two '
-                             f'arguments, not {self._num_args}') 
-        
-        
+            raise ValueError(
+                "`func` must be a function accepting one or two arguments, not "
+                f"{self._num_args}"
+            )
+
     def handle(self, field: FieldBase, t: float) -> None:
         """ handle data supplied to this tracker
         
@@ -87,16 +85,15 @@ class CallbackTracker(TrackerBase):
             self._callback(field, t)
 
 
-
 class ProgressTracker(TrackerBase):
     """ Tracker that shows the progress of the simulation """
-            
-    name = 'progress'
 
+    name = "progress"
 
-    @fill_in_docstring            
-    def __init__(self, interval: IntervalData = None,
-                 ndigits: int = 5, leave: bool = True):
+    @fill_in_docstring
+    def __init__(
+        self, interval: IntervalData = None, ndigits: int = 5, leave: bool = True
+    ):
         """
         Args:
             interval:
@@ -107,15 +104,14 @@ class ProgressTracker(TrackerBase):
                 shown maximally.
             leave (bool): Whether to leave the progress bar after the simulation
                 has finished (default: True)
-        """ 
+        """
         if interval is None:
             # print every second by default
             interval = RealtimeIntervals(duration=1)
-        
+
         super().__init__(interval=interval)
         self.ndigits = ndigits
         self.leave = leave
-        
 
     def initialize(self, field: FieldBase, info: InfoDict = None) -> float:
         """ initialize the tracker with information about the simulation
@@ -130,20 +126,21 @@ class ProgressTracker(TrackerBase):
             float: The first time the tracker needs to handle data
         """
         result = super().initialize(field, info)
-        
+
         # get solver information
-        controller_info = {} if info is None else info.get('controller', {})
-        
+        controller_info = {} if info is None else info.get("controller", {})
+
         # initialize the progress bar
         pb_cls = get_progress_bar_class()
-        self.progress_bar = pb_cls(total=controller_info.get('t_end'),
-                                   initial=controller_info.get('t_start', 0),
-                                   leave=self.leave)
-        self.progress_bar.set_description('Initializing')
+        self.progress_bar = pb_cls(
+            total=controller_info.get("t_end"),
+            initial=controller_info.get("t_start", 0),
+            leave=self.leave,
+        )
+        self.progress_bar.set_description("Initializing")
 
         return result
-            
-            
+
     def handle(self, field: FieldBase, t: float) -> None:
         """ handle data supplied to this tracker
         
@@ -159,9 +156,8 @@ class ProgressTracker(TrackerBase):
         else:
             t_new = t
         self.progress_bar.n = round(t_new, self.ndigits)
-        self.progress_bar.set_description('')
-                
-        
+        self.progress_bar.set_description("")
+
     def finalize(self, info: InfoDict = None) -> None:
         """ finalize the tracker, supplying additional information
 
@@ -170,47 +166,46 @@ class ProgressTracker(TrackerBase):
                 Extra information from the simulation        
         """
         super().finalize(info)
-        self.progress_bar.set_description('')
+        self.progress_bar.set_description("")
 
         # limit progress bar to 100%
-        controller_info = {} if info is None else info.get('controller', {}) 
-        t_final = controller_info.get('t_final', -np.inf)
-        t_end = controller_info.get('t_end', -np.inf)
+        controller_info = {} if info is None else info.get("controller", {})
+        t_final = controller_info.get("t_final", -np.inf)
+        t_end = controller_info.get("t_end", -np.inf)
         if t_final >= t_end and self.progress_bar.total:
             self.progress_bar.n = self.progress_bar.total
             self.progress_bar.refresh()
-        
-        if (controller_info.get('successful', False) and self.leave and
-                hasattr(self.progress_bar, 'sp')):
+
+        if (
+            controller_info.get("successful", False)
+            and self.leave
+            and hasattr(self.progress_bar, "sp")
+        ):
             # show progress bar in green if simulation was successful. We
             # need to overwrite the default behavior (and disable the
             # progress bar) since reaching steady state means the simulation
             # was successful even though it did not reach t_final
             try:
-                self.progress_bar.sp(bar_style='success')
+                self.progress_bar.sp(bar_style="success")
             except TypeError:
                 self.progress_bar.close()
             else:
                 self.disable = True
         else:
             self.progress_bar.close()
-            
-            
-    def __del__(self):
-        if hasattr(self, 'progress_bar') and not self.progress_bar.disable:
-            self.progress_bar.close()
 
+    def __del__(self):
+        if hasattr(self, "progress_bar") and not self.progress_bar.disable:
+            self.progress_bar.close()
 
 
 class PrintTracker(TrackerBase):
     """ Tracker that prints data to a stream (default: stdout) """
-    
-    name = 'print'
-    
-    
+
+    name = "print"
+
     @fill_in_docstring
-    def __init__(self, interval: IntervalData = 1,
-                 stream: IO[str] = sys.stdout):
+    def __init__(self, interval: IntervalData = 1, stream: IO[str] = sys.stdout):
         """
         
         Args:
@@ -221,8 +216,7 @@ class PrintTracker(TrackerBase):
         """
         super().__init__(interval=interval)
         self.stream = stream
-        
-        
+
     def handle(self, field: FieldBase, t: float) -> None:
         """ handle data supplied to this tracker
         
@@ -233,12 +227,11 @@ class PrintTracker(TrackerBase):
                 The associated time
         """
         data = f"c={field.data.mean():.3g}±{field.data.std():.3g}"
-            
+
         self.stream.write(f"t={t:g}, {data}\n")
         self.stream.flush()
 
 
-
 class PlotTracker(TrackerBase):
     """ Tracker that plots the state, either on screen or to a file
     
@@ -247,15 +240,19 @@ class PlotTracker(TrackerBase):
     on a cluster). The default values of this tracker are chosen with regular
     output to a file in mind.
     """
-    
+
     @fill_in_docstring
-    def __init__(self, interval: IntervalData = 1, *,
-                 title: Union[str, Callable] = 'Time: {time:g}',
-                 output_file: Optional[str] = None,
-                 movie: Union[str, Path, 'Movie'] = None,
-                 show: bool = None,
-                 plot_args: Dict[str, Any] = None,
-                 **kwargs):
+    def __init__(
+        self,
+        interval: IntervalData = 1,
+        *,
+        title: Union[str, Callable] = "Time: {time:g}",
+        output_file: Optional[str] = None,
+        movie: Union[str, Path, "Movie"] = None,
+        show: bool = None,
+        plot_args: Dict[str, Any] = None,
+        **kwargs,
+    ):
         """
         Args:
             interval:
@@ -297,60 +294,67 @@ class PlotTracker(TrackerBase):
             the `dpi` argument or the `frame_rate`. 
         """
         from ..visualization.movies import Movie  # @Reimport
-        
+
         # handle deprecated parameters
-        if 'movie_file' in kwargs:
+        if "movie_file" in kwargs:
             # Deprecated this method on 2020-06-04
             import warnings
-            warnings.warn("Argument `movie_file` is deprecated. Use `movie` "
-                          "instead.", DeprecationWarning)
+
+            warnings.warn(
+                "Argument `movie_file` is deprecated. Use `movie` instead.",
+                DeprecationWarning,
+            )
             if movie is None:
-                movie = kwargs.pop('movie_file')
-        if 'output_folder' in kwargs:
+                movie = kwargs.pop("movie_file")
+        if "output_folder" in kwargs:
             # Deprecated this method on 2020-06-04
             import warnings  # @Reimport
-            warnings.warn("Argument `output_folder` is deprecated. Use an "
-                          "instance of pde.visualization.movies.Movie with "
-                          "`image_folder` and supply it to the `movie` "
-                          "argument instead.", DeprecationWarning)
-            del kwargs['output_folder']
+
+            warnings.warn(
+                "Argument `output_folder` is deprecated. Use an "
+                "instance of pde.visualization.movies.Movie with "
+                "`image_folder` and supply it to the `movie` "
+                "argument instead.",
+                DeprecationWarning,
+            )
+            del kwargs["output_folder"]
         if kwargs:
             raise ValueError(f"Unused kwargs: {kwargs}")
-        
+
         # initialize the tracker
         super().__init__(interval=interval)
         self.title = title
         self.output_file = output_file
-        
+
         self.plot_args = {} if plot_args is None else plot_args.copy()
         # make sure the plot is only create and not shown since the context
         # handles showing the plot itself
-        self.plot_args['action'] = 'create'  
+        self.plot_args["action"] = "create"
 
-        # initialize the movie class        
+        # initialize the movie class
         if movie is None:
             self.movie: Optional[Movie] = None
             self._save_movie = False
-            
+
         elif isinstance(movie, Movie):
             self.movie = movie
             self._save_movie = False
-            
+
         elif isinstance(movie, (str, Path)):
             self.movie = Movie(filename=str(movie))
             self._save_movie = True
-            
-        else:
-            raise TypeError('Unknown type of argument `movie`: '
-                            f'{movie.__class__.__name__}')
 
-        # determine whether to show the images interactively            
+        else:
+            raise TypeError(
+                f"Unknown type of argument `movie`: {movie.__class__.__name__}"
+            )
+
+        # determine whether to show the images interactively
         if show is None:
             self.show = not (self._save_movie or self.output_file)
         else:
             self.show = show
-        
-     
+
     def initialize(self, state: FieldBase, info: InfoDict = None) -> float:
         """ initialize the tracker with information about the simulation
         
@@ -365,8 +369,8 @@ class PlotTracker(TrackerBase):
         """
         # initialize the plotting context
         from ..tools.plotting import get_plotting_context
-        self._context = get_plotting_context(title='Initializing...',
-                                             show=self.show)
+
+        self._context = get_plotting_context(title="Initializing...", show=self.show)
 
         # do the actual plotting
         with self._context:
@@ -374,32 +378,33 @@ class PlotTracker(TrackerBase):
 
         if self._context.supports_update:
             # the context supports reusing figures
-            if hasattr(state.plot, 'update_method'):
+            if hasattr(state.plot, "update_method"):
                 # the plotting method supports updating the plot
                 if state.plot.update_method is None:  # type: ignore
-                    if state.plot.mpl_class == 'axes':  # type: ignore
-                        self._update_method = 'update_ax'
-                    elif state.plot.mpl_class == 'figure':  # type: ignore
-                        self._update_method = 'update_fig'
+                    if state.plot.mpl_class == "axes":  # type: ignore
+                        self._update_method = "update_ax"
+                    elif state.plot.mpl_class == "figure":  # type: ignore
+                        self._update_method = "update_fig"
                     else:
                         mpl_class = state.plot.mpl_class  # type: ignore
-                        raise RuntimeError('Unknown mpl_class on plot method: '
-                                           f'{mpl_class}')
-                else: 
-                    self._update_method = 'update_data'
+                        raise RuntimeError(
+                            f"Unknown mpl_class on plot method: {mpl_class}"
+                        )
+                else:
+                    self._update_method = "update_data"
             else:
-                raise RuntimeError('PlotTracker does not  work since the state '
-                                   f'of type {state.__class__.__name__} does '
-                                   'not use the plot protocol of '
-                                   '`pde.tools.plotting`.')
+                raise RuntimeError(
+                    "PlotTracker does not  work since the state of type "
+                    f"{state.__class__.__name__} does not use the plot protocol of "
+                    "`pde.tools.plotting`."
+                )
         else:
-            self._update_method = 'replot'
-            
+            self._update_method = "replot"
+
         self._logger.info(f'Update method: "{self._update_method}"')
-                
+
         return super().initialize(state, info=info)
-        
-         
+
     def handle(self, state: FieldBase, t: float) -> None:
         """ handle data supplied to this tracker
         
@@ -413,38 +418,35 @@ class PlotTracker(TrackerBase):
             self._context.title = str(self.title(state, t))
         else:
             self._context.title = self.title.format(time=t)
-        
+
         # update the plot in the correct plotting context
         with self._context:
-            if self._update_method == 'update_data':
+            if self._update_method == "update_data":
                 # the state supports updating the plot data
-                update_func = getattr(state,
-                                      state.plot.update_method)  # type: ignore
+                update_func = getattr(state, state.plot.update_method)  # type: ignore
                 update_func(self._plot_reference)
-                
-            elif self._update_method == 'update_fig':
+
+            elif self._update_method == "update_fig":
                 fig = self._context.fig
                 fig.clf()  # type: ignore
                 state.plot(fig=fig, **self.plot_args)
-                
-            elif self._update_method == 'update_ax':
+
+            elif self._update_method == "update_ax":
                 fig = self._context.fig
                 fig.clf()  # type: ignore
                 ax = fig.add_subplot(1, 1, 1)  # type: ignore
                 state.plot(ax=ax, **self.plot_args)
-                
-            elif self._update_method == 'replot':
+
+            elif self._update_method == "replot":
                 state.plot(**self.plot_args)
-                
+
             else:
-                raise RuntimeError('Unknown update method '
-                                   f'`{self._update_method}`')
-                
+                raise RuntimeError(f"Unknown update method `{self._update_method}`")
+
         if self.output_file and self._context.fig is not None:
             self._context.fig.savefig(self.output_file)
         if self.movie:
             self.movie.add_figure(self._context.fig)
-          
 
     def finalize(self, info: InfoDict = None) -> None:
         """ finalize the tracker, supplying additional information
@@ -459,11 +461,10 @@ class PlotTracker(TrackerBase):
             self.movie.save()  # type: ignore
             # end recording the movie (e.g. delete temporary files)
             self.movie._end()  # type: ignore
-            
+
         if not self.show:
             self._context.close()
-    
-    
+
 
 class PlotInteractiveTracker(PlotTracker):
     """ Tracker that plots data on screen, to files, or writes a movie
@@ -473,13 +474,11 @@ class PlotInteractiveTracker(PlotTracker):
     something more suitable for interactive plotting. In particular, this
     tracker can be enabled by simply listing 'plot' as a tracker. 
     """
-     
-    name = 'plot'
-    
+
+    name = "plot"
+
     @fill_in_docstring
-    def __init__(self, interval: IntervalData = '0:03', *,
-                 show: bool = True,
-                 **kwargs):
+    def __init__(self, interval: IntervalData = "0:03", *, show: bool = True, **kwargs):
         """
         Args:
             interval:
@@ -503,10 +502,9 @@ class PlotInteractiveTracker(PlotTracker):
             plot_args (dict):
                 Extra arguments supplied to the plot call
         """
-        super().__init__(interval=interval, show=show, **kwargs)    
-    
-              
-    
+        super().__init__(interval=interval, show=show, **kwargs)
+
+
 class DataTracker(CallbackTracker):
     """ Tracker that stores custom data obtained by calling a function
     
@@ -517,11 +515,11 @@ class DataTracker(CallbackTracker):
             The actually stored data, which is a list of the objects returned by
             the callback function. 
     """
-    
+
     @fill_in_docstring
-    def __init__(self, func: Callable,
-                 interval: IntervalData = 1,
-                 filename: str = None):
+    def __init__(
+        self, func: Callable, interval: IntervalData = 1, filename: str = None
+    ):
         """ 
         Args:
             func:
@@ -548,8 +546,7 @@ class DataTracker(CallbackTracker):
         self.filename = filename
         self.times: List[float] = []
         self.data: List[Any] = []
-        
-        
+
     def handle(self, field: FieldBase, t: float) -> None:
         """ handle data supplied to this tracker
         
@@ -564,8 +561,7 @@ class DataTracker(CallbackTracker):
             self.data.append(self._callback(field))
         else:
             self.data.append(self._callback(field, t))
-        
-        
+
     def finalize(self, info: InfoDict = None) -> None:
         """ finalize the tracker, supplying additional information
 
@@ -576,8 +572,7 @@ class DataTracker(CallbackTracker):
         super().finalize(info)
         if self.filename:
             self.to_file(self.filename)
-        
-        
+
     @property
     def dataframe(self) -> "pandas.DataFrame":
         """ :class:`pandas.DataFrame`: the data in a dataframe
@@ -588,12 +583,12 @@ class DataTracker(CallbackTracker):
         column 'time'.
         """
         import pandas as pd
+
         df = pd.DataFrame(self.data)
         # insert the times and use them as an index
-        df.insert(0, 'time', self.times)
+        df.insert(0, "time", self.times)
         return df
-    
-    
+
     def to_file(self, filename: str, **kwargs):
         r""" store data in a file
         
@@ -609,23 +604,23 @@ class DataTracker(CallbackTracker):
                 Additional parameters may be supported for some formats 
         """
         extension = os.path.splitext(filename)[1].lower()
-        if extension == '.pickle':
-            # default 
+        if extension == ".pickle":
+            # default
             import pickle
+
             with open(filename, "wb") as fp:
                 pickle.dump((self.times, self.data), fp, **kwargs)
-            
-        elif extension == '.csv':
+
+        elif extension == ".csv":
             self.dataframe.to_csv(filename, **kwargs)
-        elif extension == '.json':
+        elif extension == ".json":
             self.dataframe.to_json(filename, **kwargs)
-        elif extension in {'.xls', '.xlsx'}:
+        elif extension in {".xls", ".xlsx"}:
             self.dataframe.to_excel(filename, **kwargs)
         else:
-            raise ValueError(f'Unsupported file extension `{extension}`')
-            
-            
-            
+            raise ValueError(f"Unsupported file extension `{extension}`")
+
+
 class SteadyStateTracker(TrackerBase):
     """ Tracker that interrupts the simulation once steady state is reached
     
@@ -633,13 +628,12 @@ class SteadyStateTracker(TrackerBase):
     case when the derivative is close to zero.
     """
 
-    name = 'steady_state'
-
+    name = "steady_state"
 
     @fill_in_docstring
-    def __init__(self, interval: IntervalData = None,
-                 atol: float = 1e-8,
-                 rtol: float = 1e-5):
+    def __init__(
+        self, interval: IntervalData = None, atol: float = 1e-8, rtol: float = 1e-5
+    ):
         """
         Args:
             interval:
@@ -650,15 +644,14 @@ class SteadyStateTracker(TrackerBase):
                 simulation
             rtol (float): Relative tolerance that must be reached to abort the
                 simulation
-        """ 
+        """
         if interval is None:
             interval = RealtimeIntervals(duration=1)
         super().__init__(interval=interval)
-        self.atol = atol 
+        self.atol = atol
         self.rtol = rtol
         self._last_data = None
-        
-        
+
     def handle(self, field: FieldBase, t: float) -> None:
         """ handle data supplied to this tracker
         
@@ -672,21 +665,19 @@ class SteadyStateTracker(TrackerBase):
             # scale with dt to make test independent of dt
             atol = self.atol * self.interval.dt
             rtol = self.rtol * self.interval.dt
-            if np.allclose(self._last_data, field.data,
-                           rtol=rtol, atol=atol, equal_nan=True):
-                raise FinishedSimulation('Reached stationary state')
-            
+            if np.allclose(
+                self._last_data, field.data, rtol=rtol, atol=atol, equal_nan=True
+            ):
+                raise FinishedSimulation("Reached stationary state")
+
         self._last_data = field.data.copy()  # store data from last timestep
-            
 
 
 class RuntimeTracker(TrackerBase):
     """ Tracker that interrupts the simulation once a duration has passed """
 
-
     @fill_in_docstring
-    def __init__(self, max_runtime: Union[Real, str],
-                 interval: IntervalData = 1):  
+    def __init__(self, max_runtime: Union[Real, str], interval: IntervalData = 1):
         """
         Args:
             max_runtime (float or str):
@@ -699,14 +690,13 @@ class RuntimeTracker(TrackerBase):
                 {ARG_TRACKER_INTERVAL}
         """
         super().__init__(interval=interval)
-        
+
         try:
             self.max_runtime = float(max_runtime)
         except ValueError:
             td = parse_duration(str(max_runtime))
             self.max_runtime = td.total_seconds()
 
-
     def initialize(self, field: FieldBase, info: InfoDict = None) -> float:
         """ 
         Args:
@@ -720,8 +710,7 @@ class RuntimeTracker(TrackerBase):
         """
         self.max_time = time.time() + self.max_runtime
         return super().initialize(field, info)
-        
-        
+
     def handle(self, field: FieldBase, t: float) -> None:
         """ handle data supplied to this tracker
         
@@ -733,16 +722,14 @@ class RuntimeTracker(TrackerBase):
         """
         if time.time() > self.max_time:
             dt = timedelta(seconds=self.max_runtime)
-            raise FinishedSimulation(f'Reached maximal runtime of {str(dt)}')
+            raise FinishedSimulation(f"Reached maximal runtime of {str(dt)}")
+
 
-            
-            
 class ConsistencyTracker(TrackerBase):
-    """ Tracker that interrupts the simulation when the state is not finite """ 
+    """ Tracker that interrupts the simulation when the state is not finite """
+
+    name = "consistency"
 
-    name = 'consistency'
-        
-    
     @fill_in_docstring
     def __init__(self, interval: IntervalData = None):
         """
@@ -751,12 +738,11 @@ class ConsistencyTracker(TrackerBase):
                 {ARG_TRACKER_INTERVAL}
                 The default value `None` checks for consistency approximately
                 every (real) second.
-        """ 
+        """
         if interval is None:
             interval = RealtimeIntervals(duration=1)
         super().__init__(interval=interval)
-        
-        
+
     def handle(self, field: FieldBase, t: float) -> None:
         """ handle data supplied to this tracker
         
@@ -767,22 +753,20 @@ class ConsistencyTracker(TrackerBase):
                 The associated time
         """
         if not np.all(np.isfinite(field.data)):
-            raise StopIteration('Field was not finite')
-            
+            raise StopIteration("Field was not finite")
+
         self._last = field.data.copy()  # store data from last timestep
-            
 
 
 class MaterialConservationTracker(TrackerBase):
     """ Ensure that the amount of material is conserved """
 
-    name = 'material_conservation'
-
+    name = "material_conservation"
 
     @fill_in_docstring
-    def __init__(self, interval: IntervalData = 1,
-                 atol: float = 1e-4,
-                 rtol: float = 1e-4):
+    def __init__(
+        self, interval: IntervalData = 1, atol: float = 1e-4, rtol: float = 1e-4
+    ):
         """
         Args:
             interval:
@@ -793,10 +777,9 @@ class MaterialConservationTracker(TrackerBase):
                 Relative tolerance for amount deviations
         """
         super().__init__(interval=interval)
-        self.atol = atol 
+        self.atol = atol
         self.rtol = rtol
-        
-        
+
     def initialize(self, field: FieldBase, info: InfoDict = None) -> float:
         """ 
         Args:
@@ -812,10 +795,9 @@ class MaterialConservationTracker(TrackerBase):
             self._reference = np.array([f.magnitude for f in field])
         else:
             self._reference = field.magnitude  # type: ignore
-            
+
         return super().initialize(field, info)
-        
-        
+
     def handle(self, field: FieldBase, t: float) -> None:
         """ handle data supplied to this tracker
         
@@ -829,17 +811,24 @@ class MaterialConservationTracker(TrackerBase):
             mags = np.array([f.magnitude for f in field])
         else:
             mags = field.magnitude  # type: ignore
-            
+
         c = np.isclose(mags, self._reference, rtol=self.rtol, atol=self.atol)
         if not np.all(c):
             if isinstance(field, FieldCollection):
-                msg = f'Material of field {np.flatnonzero(~c)} is not conserved'
+                msg = f"Material of field {np.flatnonzero(~c)} is not conserved"
             else:
-                msg = f'Material is not conserved'
+                msg = f"Material is not conserved"
             raise StopIteration(msg)
-            
-            
-            
-__all__ = ['CallbackTracker', 'ProgressTracker', 'PrintTracker', 'PlotTracker',
-           'DataTracker', 'SteadyStateTracker', 'RuntimeTracker',
-           'ConsistencyTracker', 'MaterialConservationTracker']
+
+
+__all__ = [
+    "CallbackTracker",
+    "ProgressTracker",
+    "PrintTracker",
+    "PlotTracker",
+    "DataTracker",
+    "SteadyStateTracker",
+    "RuntimeTracker",
+    "ConsistencyTracker",
+    "MaterialConservationTracker",
+]
