@@ -1,4 +1,4 @@
-'''
+"""
 Functions for creating movies of simulation results
 
 
@@ -11,16 +11,15 @@ Functions for creating movies of simulation results
    
    
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
-'''
+"""
 
-from typing import Dict, Any
+from typing import Any, Dict
 
-from .plotting import ScalarFieldPlot, ScaleData
 from ..storage.base import StorageBase
 from ..tools.docstrings import fill_in_docstring
+from .plotting import ScalarFieldPlot, ScaleData
 
 
-    
 class Movie:
     """ Class for creating movies from matplotlib figures using ffmpeg
     
@@ -30,10 +29,9 @@ class Movie:
         so that `matplotlib` can find it. 
     """
 
-    def __init__(self, filename: str,
-                 framerate: float = 30,
-                 dpi: float = None,
-                 **kwargs):
+    def __init__(
+        self, filename: str, framerate: float = 30, dpi: float = None, **kwargs
+    ):
         r"""
         Args:
             filename (str):
@@ -53,17 +51,18 @@ class Movie:
         self.dpi = dpi
         self.kwargs = kwargs
 
-        # test whether ffmpeg is available    
+        # test whether ffmpeg is available
         from matplotlib.animation import FFMpegWriter
+
         if not FFMpegWriter.isAvailable():
-            raise RuntimeError('FFMpegWriter is not available. This is most '
-                               'likely because a suitable installation of '
-                               'FFMpeg was not found. See ffmpeg.org for how '
-                               'to install it properly on your system.')
+            raise RuntimeError(
+                "FFMpegWriter is not available. This is most likely because a suitable "
+                "installation of FFMpeg was not found. See ffmpeg.org for how to "
+                "install it properly on your system."
+            )
 
         self._writer = None
-        
-        
+
     @classmethod
     def is_available(cls) -> bool:
         """ check whether the movie infrastructure is available
@@ -72,25 +71,22 @@ class Movie:
             bool: True if movies can be created
         """
         from matplotlib.animation import FFMpegWriter
+
         return FFMpegWriter.isAvailable()  # type: ignore
-        
 
     def __enter__(self):
         return self
 
-
     def __exit__(self, exc_type, exc_value, exc_tb):
         self._end()
         return False
 
-
     def _end(self):
         """ clear up temporary things if necessary """
         if self._writer is not None:
             self._writer.finish()
         self._writer = None
 
-
     def add_figure(self, fig=None):
         """ adds the figure `fig` as a frame to the current movie
         
@@ -100,14 +96,16 @@ class Movie:
         """
         if fig is None:
             import matplotlib.pyplot as plt
+
             fig = plt.gcf()
-        
+
         if self._writer is None:
             # initialize a new writer
-            from matplotlib.animation import FFMpegWriter            
+            from matplotlib.animation import FFMpegWriter
+
             self._writer = FFMpegWriter(self.framerate, **self.kwargs)
             self._writer.setup(fig, self.filename, dpi=self.dpi)
-            
+
         else:
             # update the figure reference on a given writer, since it might have
             # changed from the last call. In particular, this will happen when
@@ -115,22 +113,22 @@ class Movie:
             self._writer.fig = fig
 
         self._writer.grab_frame()
-        
 
     def save(self):
         """ convert the recorded images to a movie using ffmpeg """
         self._end()
-    
 
 
 @fill_in_docstring
-def movie_scalar(storage: StorageBase,
-                 filename: str,
-                 scale: ScaleData = 'automatic',
-                 extras: Dict[str, Any] = None,
-                 progress: bool = True,
-                 tight: bool = False,
-                 show: bool = True) -> None:
+def movie_scalar(
+    storage: StorageBase,
+    filename: str,
+    scale: ScaleData = "automatic",
+    extras: Dict[str, Any] = None,
+    progress: bool = True,
+    tight: bool = False,
+    show: bool = True,
+) -> None:
     """ produce a movie for a simulation of a scalar field
     
     Args:
@@ -152,25 +150,27 @@ def movie_scalar(storage: StorageBase,
         show (bool):
             Flag determining whether images are shown during making the movie
     """
-    quantities = [{'title': 'Concentration', 'source': None}]
+    quantities = [{"title": "Concentration", "source": None}]
     if extras:
         for key, value in extras.items():
-            quantities.append({'title': key, 'source': value})
-    
+            quantities.append({"title": key, "source": value})
+
     # initialize the plot with the first data point
-    plot = ScalarFieldPlot.from_storage(storage, quantities=quantities,
-                                        scale=scale, tight=tight, show=show)
+    plot = ScalarFieldPlot.from_storage(
+        storage, quantities=quantities, scale=scale, tight=tight, show=show
+    )
     # make the full movie
-    plot.make_movie(storage, filename, progress=progress)    
-   
-    
-    
+    plot.make_movie(storage, filename, progress=progress)
+
+
 @fill_in_docstring
-def movie_multiple(storage: StorageBase,
-                   filename: str,
-                   quantities=None,
-                   scale: ScaleData = 'automatic',
-                   progress: bool = True) -> None:
+def movie_multiple(
+    storage: StorageBase,
+    filename: str,
+    quantities=None,
+    scale: ScaleData = "automatic",
+    progress: bool = True,
+) -> None:
     """ produce a movie for a simulation with n components
 
     Args:
@@ -186,7 +186,5 @@ def movie_multiple(storage: StorageBase,
         progress (bool):
             Flag determining whether the progress of making the movie is shown.
     """
-    plot = ScalarFieldPlot.from_storage(storage, quantities=quantities,
-                                        scale=scale)
+    plot = ScalarFieldPlot.from_storage(storage, quantities=quantities, scale=scale)
     plot.make_movie(storage, filename, progress=progress)
-
